# Unix SMB/CIFS implementation.
# Copyright Volker Lendecke <vl@samba.org> 2022
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from samba.samba3 import libsmb_samba_internal as libsmb
from samba import reparse_symlink
from samba import (ntstatus,NTSTATUSError)
from samba.dcerpc import security as sec
import samba.tests.libsmb

class Smb2SymlinkTests(samba.tests.libsmb.LibsmbTests):

    def connections(self, smb1share=None, smb2share=None):
        if not smb1share:
            smb1share = samba.tests.env_get_var_value(
                "SMB1_SHARE", allow_missing=True)
            if not smb1share:
                smb1share = "nosymlinks_smb1allow"

        try:
            smb1 = libsmb.Conn(
                self.server_ip,
                smb1share,
                self.lp,
                self.creds,
                force_smb1=True)
        except NTSTATUSError as e:
            if e.args[0] != ntstatus.NT_STATUS_CONNECTION_RESET:
                raise
        smb1.smb1_posix()

        if not smb2share:
            smb2share = samba.tests.env_get_var_value(
                "SMB2_SHARE", allow_missing=True)
            if not smb2share:
                smb2share = "nosymlinks"

        smb2 = libsmb.Conn(
            self.server_ip,
            smb2share,
            self.lp,
            self.creds)
        return (smb1, smb2)

    def create_symlink(self, conn1, conn2, target, symlink,
                       expect_tgt=None):

        if expect_tgt is None:
            expect_tgt = target

        self.clean_file(conn1, symlink)
        if (conn1.protocol() < libsmb.PROTOCOL_SMB2_02 and
            conn1.have_posix()):
            conn1.smb1_symlink(target, symlink)
        else:
            flags = 0 if target[0]=='/' else 1
            syml = conn1.create(
                symlink,
                DesiredAccess=sec.SEC_FILE_READ_ATTRIBUTE|
                sec.SEC_FILE_WRITE_ATTRIBUTE|
                sec.SEC_STD_DELETE,
                FileAttributes=libsmb.FILE_ATTRIBUTE_NORMAL,
                CreateDisposition=libsmb.FILE_OPEN_IF,
                CreateOptions=libsmb.FILE_OPEN_REPARSE_POINT)
            b = reparse_symlink.symlink_put(target, target, 0, 1)
            conn1.fsctl(syml, libsmb.FSCTL_SET_REPARSE_POINT, b, 0)
            conn1.close(syml)

        fd = conn2.create(symlink,
                          DesiredAccess=sec.SEC_FILE_WRITE_ATTRIBUTE,
                          CreateOptions=libsmb.FILE_OPEN_REPARSE_POINT,
                          CreateDisposition=libsmb.FILE_OPEN)
        blob = conn2.fsctl(fd, libsmb.FSCTL_GET_REPARSE_POINT, b'', 1024)
        conn2.close(fd)

        (tag,(subst,_,_,_)) = reparse_symlink.get(blob)
        self.assertEqual(tag, "IO_REPARSE_TAG_SYMLINK")
        self.assertEqual(expect_tgt, subst)

    def assert_symlink_exception(self, e, expect):
        self.assertEqual(e.args[0], ntstatus.NT_STATUS_STOPPED_ON_SYMLINK)
        for k,v in expect.items():
            if (k == "flags"):
                # Ignore symlink trust flags for now
                expected = v & ~libsmb.SYMLINK_TRUST_MASK
                got = e.args[2].get(k) & ~libsmb.SYMLINK_TRUST_MASK
                self.assertEqual((k,got), (k,expected))
            else:
                self.assertEqual((k,e.args[2].get(k)), (k,v))

    def test_symlinkerror_directory(self):
        """Test a symlink in a nonterminal path component"""
        (smb1,smb2) = self.connections()
        symlink="syml"
        target="foo"
        suffix="bar"

        self.create_symlink(smb1, smb2, target, symlink);

        with self.assertRaises(NTSTATUSError) as e:
            fd = smb2.create_ex(f'{symlink}\\{suffix}')

        self.assert_symlink_exception(
            e.exception,
            { 'unparsed_path_length' : len(suffix)+1,
              'substitute_name' : target,
              'print_name' : target,
              'flags' : 0x20000001 })

        self.clean_file(smb1, symlink)

    def test_symlinkerror_file(self):
        """Test a simple symlink in a terminal path"""
        (smb1,smb2) = self.connections()
        symlink="syml"
        target="foo"

        self.create_symlink(smb1, smb2, target, symlink);

        with self.assertRaises(NTSTATUSError) as e:
            fd = smb2.create_ex(f'{symlink}')

        self.assert_symlink_exception(
                e.exception,
            { 'unparsed_path_length' : 0,
              'substitute_name' : target,
              'print_name' : target,
              'flags' : 0x20000001 })

        self.clean_file(smb1, symlink)

    def test_symlinkerror_absolute_outside_share(self):
        """
        Test symlinks to outside of the share
        We return the contents 1:1
        """
        (smb1,smb2) = self.connections()
        symlink="syml"

        for target in ["/etc", "//foo/bar", "/"]:

            self.create_symlink(smb1, smb2, target, symlink)

            with self.assertRaises(NTSTATUSError) as e:
                fd = smb2.create_ex(f'{symlink}')

            self.assert_symlink_exception(
                e.exception,
                { 'unparsed_path_length' : 0,
                  'substitute_name' : target,
                  'print_name' : target,
                  'flags' : 0 })

            self.clean_file(smb1, symlink)

    def test_symlinkerror_absolute_inshare(self):
        """Test an absolute symlink inside the share"""
        (smb1,smb2) = self.connections()
        symlink="syml"

        localpath=samba.tests.env_get_var_value("LOCAL_PATH")
        shareroot=f'{localpath}/nosymlinks'
        rel_dest="dst"
        target=f'{shareroot}/{rel_dest}'

        self.create_symlink(smb1, smb2, target, symlink, rel_dest)

        with self.assertRaises(NTSTATUSError) as e:
            fd = smb2.create_ex(f'{symlink}')

        self.assert_symlink_exception(
            e.exception,
            { 'unparsed_path_length' : 0,
              'substitute_name' : rel_dest,
              'print_name' : rel_dest,
              'flags' : 1 })

        self.clean_file(smb1, symlink)

    def test_symlink_reparse_data_buffer_parse(self):
        """Test parsing a symlink reparse buffer coming from Windows"""

        buf = (b'\x0c\x00\x00\xa0\x18\x00\x00\x00'
               b'\x06\x00\x06\x00\x00\x00\x06\x00'
               b'\x01\x00\x00\x00\x62\x00\x61\x00'
               b'\x72\x00\x62\x00\x61\x00\x72\x00')

        try:
            (tag,syml) = reparse_symlink.get(buf)
        except:
            self.fail("Could not parse symlink buffer")

        self.assertEqual(tag, "IO_REPARSE_TAG_SYMLINK")
        self.assertEqual(syml, ('bar', 'bar', 0, 1))

    def test_bug15505(self):
        """Test an absolute intermediate symlink inside the share"""
        (smb1,smb2) = self.connections(smb1share="tmp",smb2share="tmp")
        symlink="syml"

        localpath=samba.tests.env_get_var_value("LOCAL_PATH")

        smb1.mkdir("sub")
        self.addCleanup(self.clean_file, smb1, "sub")

        self.create_symlink(smb1, smb2, f'{localpath}/sub1', "sub/lnk")
        self.addCleanup(self.clean_file, smb1, "sub/lnk")

        smb1.mkdir("sub1")
        self.addCleanup(self.clean_file, smb1, "sub1")

        fd = smb1.create("sub1/x", CreateDisposition=libsmb.FILE_CREATE);
        smb1.close(fd)
        self.addCleanup(self.clean_file, smb1, "sub1/x")

        fd = smb2.create("sub\\lnk\\x")
        smb2.close(fd)

if __name__ == '__main__':
    import unittest
    unittest.main()



# Generate test vectors for Windows LZ77 Huffman compression.
#
# Copyright (c) 2022 Catalyst IT
#
# GPLv3+.
#
# This uses the Python ctypes module to access the lower level RTL
# compression functions.

import sys
import argparse
from ctypes import create_string_buffer, byref, windll
from ctypes.wintypes import USHORT, ULONG, LONG, PULONG, LPVOID, CHAR
NTSTATUS = LONG


METHODS = {
    'LZNT1': 2,
    'XPRESS_PLAIN': 3,
    'XPRESS_HUFF': 4,
    '2': 2,
    '3': 3,
    '4': 4
}


class RtlError(Exception):
    pass


def ntstatus_check(status, f, args):
    # 0x117 is STATUS_BUFFER_ALL_ZEROS
    status &= 0xffffffff
    if status in (0, 0x117):
        return status
    msg = {
        0xC0000023: "buffer too small",
        0xC0000242: "bad compression data",
    }.get(status, '')

    raise RtlError(f'NTSTATUS: {status:08X} {msg}')


def wrap(f, result, *args):
    f.restype = result
    f.argtypes = args
    f.errcheck = ntstatus_check
    return f


CompressBuffer = wrap(windll.ntdll.RtlCompressBuffer, NTSTATUS,
                      USHORT, LPVOID, ULONG, LPVOID, ULONG, ULONG, PULONG,
                      LPVOID)


GetCompressionWorkSpaceSize = wrap(windll.ntdll.RtlGetCompressionWorkSpaceSize,
                                   NTSTATUS,
                                   USHORT, PULONG, PULONG)


DecompressBufferEx = wrap(windll.ntdll.RtlDecompressBufferEx,
                          NTSTATUS,
                          USHORT, LPVOID, ULONG, LPVOID, ULONG, PULONG, LPVOID)


def compress(data, format, effort=0):
    flags = USHORT(format | effort)
    workspace_size = ULONG(0)
    fragment_size = ULONG(0)
    comp_len = ULONG(0)
    GetCompressionWorkSpaceSize(flags,
                                byref(workspace_size),
                                byref(fragment_size))
    workspace = create_string_buffer(workspace_size.value)
    output_len = len(data) * 9 // 8 + 260
    output_buf = bytearray(output_len)
    CompressBuffer(flags,
                   (CHAR * 1).from_buffer(data), len(data),
                   (CHAR * 1).from_buffer(output_buf), output_len,
                   4096,
                   byref(comp_len),
                   workspace)
    return output_buf[:comp_len.value]


def decompress(data, format, target_size=None):
    flags = USHORT(format)
    workspace_size = ULONG(0)
    fragment_size = ULONG(0)
    decomp_len = ULONG(0)
    GetCompressionWorkSpaceSize(flags,
                                byref(workspace_size),
                                byref(fragment_size))
    workspace = create_string_buffer(workspace_size.value)
    if target_size is None:
        output_len = len(data) * 10
    else:
        output_len = target_size
    output_buf = bytearray(output_len)

    DecompressBufferEx(format,
                       (CHAR * 1).from_buffer(output_buf), len(output_buf),
                       (CHAR * 1).from_buffer(data), len(data),
                       byref(decomp_len),
                       workspace)
    return output_buf[:decomp_len.value]


def main():
    if sys.getwindowsversion().major < 7:
        print("this probably won't work on your very old version of Windows\n"
              "but we'll try anyway!", file=sys.stderr)

    parser = argparse.ArgumentParser()
    parser.add_argument('-d', '--decompress', action='store_true',
                        help='decompress instead of compress')
    parser.add_argument('-m', '--method', default='XPRESS_HUFF',
                        choices=list(METHODS.keys()),
                        help='use this compression method')
    parser.add_argument('-e', '--extra-effort', action='store_true',
                        help='use extra effort to compress')

    parser.add_argument('-s', '--decompressed-size', type=int,
                        help=('decompress to this size '
                              '(required for XPRESS_HUFF'))

    parser.add_argument('-o', '--output',
                        help='write to this file')
    parser.add_argument('-i', '--input',
                        help='read data from this file')

    args = parser.parse_args()

    method = METHODS[args.method]

    if all((args.decompress,
            args.decompressed_size is None,
            method == 4)):
        print("a size is required for XPRESS_HUFF decompression")
        sys.exit(1)

    with open(args.input, 'rb') as f:
        data = bytearray(f.read())

    if args.decompress:
        output = decompress(data, method, args.decompressed_size)
    else:
        effort = 1 if args.extra_effort else 0
        output = compress(data, method, effort)

    with open(args.output, 'wb') as f:
        f.write(output)


main()



# Unix SMB/CIFS implementation.
#
# GroupManagedServiceAccount model.
#
# Copyright (C) Catalyst.Net Ltd. 2024
#
# Written by Rob van der Linde <rob@catalyst.net.nz>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from samba.dcerpc import security
from samba.dsdb import DS_GUID_MANAGED_SERVICE_ACCOUNTS_CONTAINER

from .computer import Computer
from .constants import GROUP_MSA_MEMBERSHIP_DEFAULT
from .exceptions import FieldError
from .fields import BinaryField, EnumField, IntegerField, SDDLField, StringField
from .types import SupportedEncryptionTypes


class GroupManagedServiceAccount(Computer):
    """A GroupManagedServiceAccount is a type of Computer which is also a User."""
    managed_password_interval = IntegerField("msDS-ManagedPasswordInterval",
                                             default=30)
    dns_host_name = StringField("dNSHostName")
    group_msa_membership = SDDLField("msDS-GroupMSAMembership",
                                     default=GROUP_MSA_MEMBERSHIP_DEFAULT)
    managed_password_id = BinaryField("msDS-ManagedPasswordId",
                                      readonly=True, hidden=True)
    managed_password_previous_id = BinaryField("msDS-ManagedPasswordPreviousId",
                                               readonly=True, hidden=True)
    supported_encryption_types = EnumField("msDS-SupportedEncryptionTypes",
                                           SupportedEncryptionTypes)

    @staticmethod
    def get_base_dn(samdb):
        """Return base Dn for Managed Service Accounts.

        :param samdb: SamDB connection
        :return: Dn to use for searching
        """
        return samdb.get_wellknown_dn(samdb.get_default_basedn(),
                                      DS_GUID_MANAGED_SERVICE_ACCOUNTS_CONTAINER)

    @staticmethod
    def get_object_class():
        return "msDS-GroupManagedServiceAccount"

    @property
    def trustees(self):
        """Returns list of trustees from the msDS-GroupMSAMembership field.

        :return: list of SID strings
        """
        allowed = []

        # Make sure to exclude DENY aces.
        for ace in self.group_msa_membership.dacl.aces:
            if ((ace.access_mask & security.SEC_ADS_READ_PROP)
                    and ace.type == security.SEC_ACE_TYPE_ACCESS_ALLOWED):
                allowed.append(str(ace.trustee))
            else:
                raise FieldError(
                    "Cannot be represented as a simple list (try viewing as SDDL)",
                    field=GroupManagedServiceAccount.group_msa_membership)

        return allowed

    def add_trustee(self, trustee: str):
        """Adds the User `trustee` to group_msa_membership.

        Checking if the trustee already has access is the responsibility
        of the caller.

        :param trustee: SID of trustee to add
        """
        aces = self.group_msa_membership.dacl.aces

        ace = security.ace()
        ace.type = security.SEC_ACE_TYPE_ACCESS_ALLOWED
        ace.trustee = security.dom_sid(trustee)
        ace.access_mask = security.SEC_ADS_GENERIC_ALL
        aces.append(ace)

        # Because aces is a copy this is necessary, also setting num_aces.
        self.group_msa_membership.dacl.aces = aces
        self.group_msa_membership.dacl.num_aces = len(aces)

    def remove_trustee(self, trustee: str):
        """Removes the User 'trustee' from group_msa_membership.

        If the trustee doesn't have access already then do nothing.

        :param trustee: SID of trustee to remove
        """
        aces = self.group_msa_membership.dacl.aces

        for ace in aces:
            if trustee == str(ace.trustee):
                self.group_msa_membership.dacl_del_ace(ace)
                break



#
# Unix SMB/CIFS implementation.
# Copyright Ralph Boehme <slow@samba.org> 2019
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

"""Tests for samba.dcerpc.mdssvc"""

import os
import time
import threading
import logging
import json
from http.server import HTTPServer, BaseHTTPRequestHandler
import samba
from samba.dcerpc import mdssvc
from samba.tests import RpcInterfaceTestCase
from samba.samba3 import mdscli
from samba.logger import get_samba_logger

logger = get_samba_logger(name=__name__)

testfiles = [
    "foo",
    "bar",
    "x+x",
    "x*x",
    "x=x",
    "x'x",
    "x?x",
    "x\"x",
    "x\\x",
    "x(x",
    "x x",
]

class MdssvcHTTPRequestHandler(BaseHTTPRequestHandler):
    def do_POST(self):
        content_length = int(self.headers['content-length'])
        body = self.rfile.read(content_length)

        actual_json = json.loads((body))
        expected_json = json.loads(self.server.json_in)

        if actual_json != expected_json:
            logger.error("Bad request, expected:\n%s\nGot:\n%s\n" % (expected_json, actual_json))
            self.send_error(400,
                            "Bad request",
                            "Expected: %s\n"
                            "Got: %s\n" %
                            (expected_json, actual_json))
            return

        resp = bytes(self.server.json_out, encoding="utf-8")

        self.send_response(200)
        self.send_header('content-type', 'application/json; charset=UTF-8')
        self.send_header('content-length', len(resp))
        self.end_headers()
        self.wfile.write(resp)

class MdssvcTests(RpcInterfaceTestCase):

    def setUp(self):
        super().setUp()

        # Build the global inject file path
        server_conf = samba.tests.env_get_var_value("SERVERCONFFILE")
        server_conf_dir = os.path.dirname(server_conf)
        self.global_inject = os.path.join(server_conf_dir, "global_inject.conf")

        self.pipe = mdssvc.mdssvc('ncacn_np:fileserver[/pipe/mdssvc]', self.get_loadparm())

        self.server = HTTPServer(('10.53.57.35', 8080),
                                 MdssvcHTTPRequestHandler,
                                 bind_and_activate=False)

        self.t = threading.Thread(target=MdssvcTests.http_server, args=(self,))
        self.t.setDaemon(True)
        self.t.start()
        self.sharepath = os.environ["LOCAL_PATH"]
        time.sleep(1)

        conn = mdscli.conn(self.pipe, 'spotlight', '/foo')
        self.fakepath = conn.sharepath()
        conn.disconnect(self.pipe)

        for file in testfiles:
            f = open("%s/%s" % (self.sharepath, file), "w")
            f.close()

    def tearDown(self):
        super().tearDown()
        for file in testfiles:
            os.remove("%s/%s" % (self.sharepath, file))

    def http_server(self):
        self.server.server_bind()
        self.server.server_activate()
        self.server.serve_forever()

    def run_test(self, query, expect, json_in, json_out):
        self.server.json_in = json_in.replace("%BASEPATH%", self.sharepath)
        self.server.json_out = json_out.replace("%BASEPATH%", self.sharepath)

        self.conn = mdscli.conn(self.pipe, 'spotlight', '/foo')
        search = self.conn.search(self.pipe, query, self.fakepath)

        # Give it some time, the get_results() below returns immediately
        # what's available, so if we ask to soon, we might get back no results
        # as the server is still processing the request
        time.sleep(1)

        results = search.get_results(self.pipe)
        self.assertEqual(results, expect)

        search.close(self.pipe)
        self.conn.disconnect(self.pipe)

    def test_mdscli_search(self):
        exp_json_query = r'''{
          "from": 0, "size": 50, "_source": ["path.real"],
          "query": {
            "bool": {
              "filter": [
                {
                  "prefix": {
                    "path.real": "%BASEPATH%/"
                  }
                }
              ],
              "must": [
                {
                  "query_string": {
                    "query": "samba*",
                    "fields": ["file.filename", "content"]
                  }
                }
              ]
            }
          }
        }'''
        fake_json_response = '''{
          "hits" : {
            "total" : { "value" : 2},
            "hits" : [
              {"_source" : {"path" : {"real" : "%BASEPATH%/foo"}}},
              {"_source" : {"path" : {"real" : "%BASEPATH%/bar"}}}
            ]
          }
        }'''
        exp_results = ["foo", "bar"]
        self.run_test('*=="samba*"', exp_results, exp_json_query, fake_json_response)

    def test_mdscli_search_escapes(self):
        sl_query = (
            r'kMDItemFSName=="x+x"||'
            r'kMDItemFSName=="x\*x"||'
            r'kMDItemFSName=="x=x"||'
            'kMDItemFSName=="x\'x"||'
            r'kMDItemFSName=="x?x"||'
            r'kMDItemFSName=="x x"||'
            r'kMDItemFSName=="x(x"||'
            r'kMDItemFSName=="x\"x"||'
            r'kMDItemFSName=="x\\x"'
        )
        exp_json_query = r'''{
          "from": 0, "size": 50, "_source": ["path.real"],
          "query": {
            "bool": {
              "filter": [
                {
                  "prefix": {
                    "path.real": "%BASEPATH%/"
                  }
                }
              ],
              "must": [
                {
                  "query_string": {
                    "query": "file.filename:x\\+x OR file.filename:x\\*x OR file.filename:x=x OR file.filename:x'x OR file.filename:x\\?x OR file.filename:x\\ x OR file.filename:x\\(x OR file.filename:x\\\"x OR file.filename:x\\\\x",
                    "fields": ["file.filename", "content"]
                  }
                }
              ]
            }
          }
        }'''
        fake_json_response = r'''{
          "hits" : {
            "total" : {"value" : 9},
            "hits" : [
              {"_source" : {"path" : {"real" : "%BASEPATH%/x+x"}}},
              {"_source" : {"path" : {"real" : "%BASEPATH%/x*x"}}},
              {"_source" : {"path" : {"real" : "%BASEPATH%/x=x"}}},
              {"_source" : {"path" : {"real" : "%BASEPATH%/x'x"}}},
              {"_source" : {"path" : {"real" : "%BASEPATH%/x?x"}}},
              {"_source" : {"path" : {"real" : "%BASEPATH%/x x"}}},
              {"_source" : {"path" : {"real" : "%BASEPATH%/x(x"}}},
              {"_source" : {"path" : {"real" : "%BASEPATH%/x\"x"}}},
              {"_source" : {"path" : {"real" : "%BASEPATH%/x\\x"}}}
            ]
          }
        }'''
        exp_results = [
            r"x+x",
            r"x*x",
            r"x=x",
            r"x'x",
            r"x?x",
            r"x x",
            r"x(x",
            "x\"x",
            r"x\x",
        ]
        self.run_test(sl_query, exp_results, exp_json_query, fake_json_response)

    def test_mdscli_search_force_substring(self):
        with open(self.global_inject, 'w') as f:
            f.write("elasticsearch:force substring search = yes\n")
        os.system("killall -SIGHUP rpcd_mdssvc")

        exp_json_query = r'''{
          "from": 0, "size": 50, "_source": ["path.real"],
          "query": {
            "bool": {
              "filter": [
                {
                  "prefix": {
                    "path.real": "%BASEPATH%/"
                  }
                }
              ],
              "must": [
                {
                  "query_string": {
                    "query": "*samba*",
                    "fields": ["file.filename", "content"]
                  }
                }
              ]
            }
          }
        }'''
        fake_json_response = '''{
          "hits" : {
            "total" : { "value" : 2},
            "hits" : [
              {"_source" : {"path" : {"real" : "%BASEPATH%/foo"}}},
              {"_source" : {"path" : {"real" : "%BASEPATH%/bar"}}}
            ]
          }
        }'''
        exp_results = ["foo", "bar"]
        self.run_test('*=="samba*"', exp_results, exp_json_query, fake_json_response)

        os.remove(self.global_inject)
        os.system("killall -SIGHUP rpcd_mdssvc")



# Unix SMB/CIFS implementation.
# mdb util helpers
#
# Copyright (C) Andrew Bartlett <abartlet@samba.org> 2018
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import subprocess
import os
from samba.netcmd import CommandError


def mdb_copy(file1, file2):
    """Copy mdb file using mdb_copy utility and rename it
    """
    # Find the location of the mdb_copy tool
    dirs = os.getenv('PATH').split(os.pathsep)
    found = False
    for d in dirs:
        toolpath = os.path.join(d, "mdb_copy")
        if os.path.exists(toolpath):
            found = True
            break

    if not found:
        raise CommandError("mdb_copy not found. "
                           "You may need to install the lmdb-utils package")

    mdb_copy_cmd = [toolpath, "-n", file1, "%s.copy.mdb" % file1]
    status = subprocess.check_call(mdb_copy_cmd, close_fds=True, shell=False)

    os.rename("%s.copy.mdb" % file1, file2)
